import sys


class Worker:
    def __init__(self, config):
        """
        type driver: str
        """
        self.id = config['id']
        self.endless = config['endless'] if 'endless' in config else False
        self.run_priority = int(config['run_priority']) if 'run_priority' in config else sys.maxint  # noqa
        self.config = config
        driver_klass = config['driver_class']
        driver_config =\
            config['driver_config'] if 'driver_config' in config else {}
        driver_config['id'] = config['id']
        self.driver = driver_klass(driver_config)

    def init(self):
        self.driver.init()

    def start_test(self):
        self.driver.start_test()

    def stop_test(self):
        self.driver.stop_test()

    def status(self):
        return self.driver.status()

    def collect_results(self):
        return self.driver.collect_results()



import threading

from status import WorkerStatus
from worker import Worker


class ScaleHub(object):
    def __init__(self, config):
        """
        type config: dict
        """
        self.workers_config = config['workers']
        self.workers = []
        for worker_config in self.workers_config:
            self.workers.append(Worker(worker_config))

    def go(self):
        self.init_workers()
        self.start_test()

    def is_test_completed(self):
        return reduce(
            lambda prev, curr: prev and (curr.endless or curr.status() == WorkerStatus.test_completed),  # noqa
            self.workers,
            True
        )

    def init_workers(self):
        for worker in self.workers:
            threading.Thread(target=worker.init).start()

        running_workers = range(0, len(self.workers))
        while len(running_workers) > 0:
            for i in running_workers:
                if self.workers[i].status() == WorkerStatus.init_completed:
                    running_workers.remove(i)

    def start_test(self):
        for worker in sorted(self.workers, key=lambda x: x.run_priority):
            threading.Thread(target=worker.start_test).start()

    def stop_endless(self):
        for worker in self.workers:
            if worker.endless:
                worker.stop_test()

    def collect_results(self):
        results = {}
        for worker in self.workers:
            results[worker.id] = worker.collect_results()
        return results



"""A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
"""

from setuptools import setup
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='scalehub',

    version='1.0.0b1',

    description='Test automation framework',

    long_description=long_description,

    url='https://github.com/scalehub/scalehub',

    author='PyScale team',
    author_email='wedonthaveemailyet@pyscale.com',

    license='MIT',

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Testing',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
    ],

    keywords='testing automation',

    packages=['scalehub'],

    install_requires=[],

    extras_require={
        'test': ['mock>=2.0', 'pytest>=3.0'],
    },
)
